%---------------------------------als_test--------------------------------%
% This file tests the ALS algorithm.
%
% External functions:
%
% - kr.m              : Khatri-Rao product;
% - als.m             : ALS algorithm;
% - amb_correct.m     : Matching Permutation for two CP models.
%
% List of updates                 -     22/07/2015  -     R. C. Farias 
%                                       Creation of the file
%-------------------------------------------------------------------------%

%--------------Cleaning the workspace and loading functions---------------%
clc;
clear     all;
close     all;
%-------------------------------------------------------------------------%

%-----------------------------Model parameters----------------------------%
% 3-way block dimensions
dim       =     [100 100 100];
% Number of components
R         =     5;
% Standard deviations of the measurements noise
sigma_n   =     0.00001;
% Number of iterations
iter      =     1000;
%-------------------------------------------------------------------------%

%------------------------------Tensor generation--------------------------%
% Random factors generation
A         =     randn(dim(1),R);
B         =     randn(dim(2),R);
C         =     randn(dim(3),R);
% Normalization
A         =     A.*repmat(1./sqrt(sum(A.^2)),dim(1),1);
B         =     B.*repmat(1./sqrt(sum(B.^2)),dim(2),1);
%-------------------------------------------------------------------------%

%-------------------------------------------------------------------------%
%Noisy measurements
% Noisy tensor
Y           =     reshape(A*transpose(kr(C,B)),dim)+...
                  sigma_n*randn(dim);
            
%-----------------------------Initialization------------------------------%
A_0         =     randn(size(A));
B_0         =     randn(size(B));
A_0         =     A_0.*repmat(1./sqrt(sum(A_0.^2)),dim(1),1);
B_0         =     B_0.*repmat(1./sqrt(sum(B_0.^2)),dim(2),1);
C_0         =     randn(size(C));
%-------------------------------------------------------------------------%

%-------------------------------------------------------------------------%

%--------------------------ALS updated version----------------------------%
tic
% ALS
[A_als,B_als,C_als,err_als_vec]   =     als(Y,iter,A_0,B_0,C_0);
toc
% Resulting factors
% Correcting permutation and sign ambiguities
[A_als,B_als,C_als]   =     amb_correct(A_als,B_als,C_als,A,B,C);
% Column of the factors to be displayed
r = 2;
subplot(3,1,1)
plot(A(:,r),'k'),hold, plot(A_als(:,r),'bd'), legend(...
    'Ground truth column','Estimated Column'), ylabel('A')
title(['Column ',num2str(r)])
subplot(3,1,2)
plot(B(:,r),'k'),hold, plot(B_als(:,r),'bd'), ylabel('B')
subplot(3,1,3)
plot(C(:,r),'k'),hold, plot(C_als(:,r),'bd'), ylabel('C')
%-------------------------------------------------------------------------%

%-------------------------------------------------------------------------%

